using System.Collections.Generic;

namespace Plux.Framework {
  public class DependentPlugAttribute : FrameworkPlugAttribute {
    public string Slot;
    public string[] Slots;
    public DependentPlugAttribute(string plug) : base(plug) {}
  }

  public class DependentPlug {
    readonly List<SlotInfo> cancelledSlots = new List<SlotInfo>();
    readonly PlugInfo plug;
    readonly List<SlotInfo> slots = new List<SlotInfo>();
    public DependentPlug(PlugInfo plug, SlotInfo[] slots) {
      this.plug = plug;
      plug.Plugging += Plug_Plugging;
      foreach (SlotInfo slot in slots) {
        this.slots.Add(slot);
        slot.Plugged += Slot_Plugged;
      }
    }
    public void Detach() {
      plug.Plugging -= Plug_Plugging;
      foreach(SlotInfo slot in slots)
        slot.Plugged -= Slot_Plugged;
    }
    void Plug_Plugging(object s, CancelPlugEventArgs args) {
      foreach(SlotInfo slot in slots) {
        if(slot.PluggedPlugInfos.Count == 0) {
          args.Cancel = true;
          slot.Open();
        }
        if(args.Cancel)
          cancelledSlots.Add(args.SlotInfo);
      }
    }
    void Slot_Plugged(object sender, PlugEventArgs args) {
      slots.Remove(args.SlotInfo);
      args.SlotInfo.Plugged -= Slot_Plugged;
      if(slots.Count == 0)
        Runtime.TaskQueue.QueueEmptied += QueueEmptied;
    }
    public void QueueEmptied(object s, TaskEventArgs args) {
      Runtime.TaskQueue.QueueEmptied -= QueueEmptied;
      foreach(SlotInfo slot in cancelledSlots)
        plug.Plug(slot);
    }
  }

  public class DependentPlugStartup : PlugStartup<DependentPlugAttribute> {
    static readonly Dictionary<PlugInfo, DependentPlug> data =
      new Dictionary<PlugInfo, DependentPlug>();
    public override void Register(ExtensionTypeInfo extType) {}
    public override void Unregister(ExtensionTypeInfo extType) {}
    public override void PlugCreated(PlugInfo plug, DependentPlugAttribute attr) {
      List<SlotInfo> slots = new List<SlotInfo>();
      if(attr.Slots != null)
        foreach(string s in attr.Slots)
          slots.Add(plug.ExtensionInfo.SlotInfos[s]);
      else slots.Add(plug.ExtensionInfo.SlotInfos[attr.Slot]);
      data[plug] = new DependentPlug(plug, slots.ToArray());
    }
    public override void PlugReleased
      (PlugInfo plug, DependentPlugAttribute attr) {
      data.Remove(plug);
    }
  }
}