using System.Collections.Generic;

namespace Plux.Framework {
  public static class ExtensionInfoEx {
    public static PlugInfo CreateExtension(this PlugTypeInfo plugType, SlotInfo slot) {
      ExtensionInfo ext = plugType.ExtensionTypeInfo.CreateExtension(slot.Unique);
      return ext.PlugInfos[plugType.Name];
    }

    public static ExtensionInfo CreateExtension(this ExtensionTypeInfo extType, bool unique) {
      return unique ? extType.CreateUniqueExtension() : extType.GetSharedExtension(true);
    }

    public static object FindIncomingObjectTransitive(this ExtensionInfo ext, string slotName) {
      return FindIncomingPlugTransitive(ext, slotName).Object;
    }

    public static PlugInfo FindIncomingPlugTransitive(ExtensionInfo ext, string slotName) {
      SlotInfo slot = ext.FindProvidedSlotTransitive(slotName);
      if (slot == null) {
        return null;
      }
      return slot.PluggedPlugInfos[0].ExtensionInfo.PlugInfos[slotName];
    }

    public static SlotInfo FindProvidedSlotTransitive(this ExtensionInfo ext, string slotName) {
      if (ext.SlotInfos.Contains(slotName)) {
        return ext.SlotInfos[slotName];
      }
      foreach (SlotInfo slot in ext.SlotInfos) {
        foreach (PlugInfo plug in slot.PluggedPlugInfos) {
          SlotInfo result = plug.ExtensionInfo.FindProvidedSlotTransitive(slotName);
          if (result != null) {
            return result;
          }
        }
      }
      return null;
    }

    // TODO: Remove 
    //public static Plug[] FindPluggedPlugsInHostTransitive(this ExtensionInfo ext, string name) {
    //  List<Plug> result = new List<Plug>();
    //  result.AddRange(ext.PlugInfos.Find(name));
    //  foreach (Plug plug in ext.PlugInfos) {
    //    foreach (SlotInfo slot in plug.PluggedInSlots) {
    //      result.AddRange(slot.ExtensionInfo.FindPluggedPlugsInHostTransitive(name));
    //    }
    //  }
    //  return result.ToArray();
    //}

    // TODO: Remove
    //public static Plug FindPluggedPlugInHostTransitive(this ExtensionInfo ext, string name) {
    //  foreach (Plug plug in ext.PlugInfos) {
    //    foreach (SlotInfo slot in plug.PluggedInSlots) {
    //      if (plug.Name.Equals(name)) {
    //        return plug;
    //      }
    //      Plug result = slot.ExtensionInfo.FindPluggedPlugInHostTransitive(name);
    //      if (result != null) {
    //        return result;
    //      }
    //    }
    //  }
    //  return null;
    //}

#if (!SILVERLIGHT)
    public static List<Path> FindHostPaths(this ExtensionInfo fromExtension, SlotInfo toSlot) {
      HashSet<ExtensionInfo> visited = new HashSet<ExtensionInfo>();
      return FindHostPaths(fromExtension, toSlot, visited);
    }

    private static List<Path> FindHostPaths(ExtensionInfo fromExtension, SlotInfo toSlot, HashSet<ExtensionInfo> visited) {
      if (visited.Contains(fromExtension)) {
        return null;
      }
      visited.Add(fromExtension);

      List<Path> paths = new List<Path>();
      AddDirectPath(fromExtension, toSlot, paths);
      AddIndirectPaths(fromExtension, toSlot, visited, paths);

      visited.Remove(fromExtension);

      return paths;
    }

    private static void AddDirectPath(ExtensionInfo fromExtension, SlotInfo toSlot, List<Path> paths) {
      foreach (PlugInfo plug in fromExtension.PlugInfos) {
        foreach (SlotInfo slot in plug.PluggedInSlots) {
          if (slot == toSlot) {
            paths.Add(new Path(new Path.Connection(plug, slot)));
            break;
          }
        }
      }
    }

    private static void AddIndirectPaths(ExtensionInfo fromExtension, SlotInfo toSlot, HashSet<ExtensionInfo> visited, List<Path> paths) {
      foreach (PlugInfo plug in fromExtension.PlugInfos) {
        foreach (SlotInfo slot in plug.PluggedInSlots) {
          ExtensionInfo ext = slot.ExtensionInfo;
          foreach (Path indirectPath in FindHostPaths(ext, toSlot, visited)) {
            paths.Add(new Path(new Path.Connection(plug, slot), indirectPath));
          }
        }
      }
    }
#endif
  }
}