﻿using System;
using System.Collections.Generic;
using System.Globalization;

namespace Plux.Framework {
  public static class MenuItemParser {
    public static MenuItemPath ParseMenuItemAttributes(string menuItemString) {
      MenuItemPath path = new MenuItemPath();
      string[] mainSplit = menuItemString.Split(new[] { "##" }, StringSplitOptions.None);
      string[] menuItems = mainSplit[0].Split('~');
      foreach (string t in menuItems) {
        string[] itemAttributeStrings = t.Split('|');
        if (itemAttributeStrings.Length == 0) {
          continue;
        }
        double order;
        if (itemAttributeStrings.Length == 1 || !double.TryParse(itemAttributeStrings[1], NumberStyles.Float, CultureInfo.InvariantCulture, out order)) {
          order = double.MaxValue;
        }
        path.Add(new MenuItemAttributes(itemAttributeStrings[0], order));
      }

      if (mainSplit.Length > 1) {
        path.Icon = mainSplit[1];
      }
      return path;
    }
  }

  public class MenuItemPath : List<MenuItemAttributes> {
    public string Icon { get; internal set; }
  }

  public class MenuItemAttributes {
    public string Text { get; private set; }
    public double Order { get; private set; }
    public MenuItemAttributes(string text, double order) {
      Text = text;
      Order = order;
    }
  }
}
